@@ -1,10 +1,12 @@
 use strict;
 use warnings;
 package Net::SAML2::Binding::Redirect;
-our $VERSION = '0.56'; # TRIAL VERSION
+our $VERSION = '0.58'; # TRIAL VERSION
 
 use Moose;
 use MooseX::Types::URI qw/ Uri /;
+use Net::SAML2::Types qw(signingAlgorithm SAMLRequestType);
+use Carp qw(croak);
 
 # ABSTRACT: Net::SAML2::Binding::Redirect - HTTP Redirect binding for SAML
 
@@ -20,13 +22,48 @@ use File::Slurper qw/ read_text /;
 use URI::Encode qw/uri_decode/;
 
 
-has 'key'   => (isa => 'Str', is => 'ro', required => 1);
-has 'cert'  => (isa => 'Str', is => 'ro', required => 1);
-has 'url'   => (isa => Uri, is => 'ro', required => 1, coerce => 1);
-has 'param' => (isa => 'Str', is => 'ro', required => 1);
-has 'sig_hash' => (isa => 'Str', is => 'ro', required => 0);
-has 'sls_force_lcase_url_encoding'    => (isa => 'Bool', is => 'ro', required => 0);
-has 'sls_double_encoded_response' => (isa => 'Bool', is => 'ro', required => 0);
+has 'cert' => (isa => 'Str', is => 'ro', required => 1);
+has 'url'  => (isa => Uri, is => 'ro', required => 0, coerce => 1, predicate => 'has_url');
+has 'key'  => (isa => 'Str', is => 'ro', required => 0, predicate => 'has_key');
+
+has 'param' => (
+    isa      => SAMLRequestType,
+    is       => 'ro',
+    required => 0,
+    default  => 'SAMLRequest'
+);
+
+has 'sig_hash' => (
+    isa      => signingAlgorithm,
+    is       => 'ro',
+    required => 0,
+    default  => 'sha1'
+);
+
+has 'sls_force_lcase_url_encoding' => (
+    isa      => 'Bool',
+    is       => 'ro',
+    required => 0,
+    default  => 0
+);
+
+has 'sls_double_encoded_response' => (
+    isa      => 'Bool',
+    is       => 'ro',
+    required => 0,
+    default  => 0
+);
+
+
+sub BUILD {
+    my $self = shift;
+
+    if ($self->param eq 'SAMLRequest') {
+        croak("Need to have an URL specified") unless $self->has_url;
+        croak("Need to have a key specified") unless $self->has_key;
+    }
+    # other params don't need to have these per-se
+}
 
 
 sub sign {
@@ -45,32 +82,19 @@ sub sign {
     my $key_string = read_text($self->key);
     my $rsa_priv = Crypt::OpenSSL::RSA->new_private_key($key_string);
 
-    if ( exists $self->{ sig_hash } && grep { $_ eq $self->{ sig_hash } } ('sha224', 'sha256', 'sha384', 'sha512'))
-    {
-        if ($self->{ sig_hash } eq 'sha224') {
-            $rsa_priv->use_sha224_hash;
-        } elsif ($self->{ sig_hash } eq 'sha256') {
-            $rsa_priv->use_sha256_hash;
-        } elsif ($self->{ sig_hash } eq 'sha384') {
-            $rsa_priv->use_sha384_hash;
-        } elsif ($self->{ sig_hash } eq 'sha512') {
-            $rsa_priv->use_sha512_hash;
-        } else {
-            die "Unsupported Signing Hash";
-        }
-        $u->query_param('SigAlg', 'http://www.w3.org/2001/04/xmldsig-more#rsa-' . $self->{ sig_hash });
-    }
-    else { #$self->{ sig_hash } eq 'sha1' or something unsupported
-        $rsa_priv->use_sha1_hash;
-        $u->query_param('SigAlg', 'http://www.w3.org/2000/09/xmldsig#rsa-sha1');
-    }
+    my $method = "use_" . $self->sig_hash . "_hash";
+    $rsa_priv->$method;
+
+    $u->query_param('SigAlg',
+        $self->sig_hash eq 'sha1'
+        ? 'http://www.w3.org/2000/09/xmldsig#rsa-sha1'
+        : 'http://www.w3.org/2001/04/xmldsig-more#rsa-' . $self->sig_hash);
 
     my $to_sign = $u->query;
     my $sig = encode_base64($rsa_priv->sign($to_sign), '');
     $u->query_param('Signature', $sig);
 
-    my $url = $u->as_string;
-    return $url;
+    return $u->as_string;
 }
 
 
@@ -102,7 +126,7 @@ sub verify {
     }
 
     # Some IdPs (PingIdentity) seem to double encode the LogoutResponse URL
-    if (defined $self->sls_double_encoded_response and $self->sls_double_encoded_response == 1) {
+    if ($self->sls_double_encoded_response) {
         #if ($sigalg =~ m/%/) {
         $signed = uri_decode($u->query);
         $sig = uri_decode($sig);
@@ -116,7 +140,7 @@ sub verify {
     # What can we say about this one Microsoft Azure uses lower case in the
     # URL encoding %2f not %2F.  As it is signed as %2f the resulting signed
     # needs to change it to lowercase if the application layer reencoded the URL.
-    if (defined $self->sls_force_lcase_url_encoding and $self->sls_force_lcase_url_encoding == 1) {
+    if ($self->sls_force_lcase_url_encoding) {
         # TODO: This is a hack.
         $signed =~ s/(%..)/lc($1)/ge;
     }
@@ -164,7 +188,7 @@ Net::SAML2::Binding::Redirect - Net::SAML2::Binding::Redirect - HTTP Redirect bi
 
 =head1 VERSION
 
-version 0.56
+version 0.58
 
 =head1 SYNOPSIS
 
@@ -213,6 +237,7 @@ IdP's SSO (Single Sign Out) service url for the Redirect binding
 =item B<param>
 
 query param name to use (SAMLRequest, SAMLResponse)
+Defaults to C<SAMLRequest>.
 
 =item B<sig_hash>
 
@@ -222,7 +247,7 @@ Supported:
 
 sha1, sha224, sha256, sha384, sha512
 
-sha1 is current default but will change by version 44
+Defaults to C<sha1>.
 
 =item B<sls_force_lcase_url_encoding>
 
@@ -238,6 +263,8 @@ The double encoding requires it to be decoded prior to processing.
 
 =back
 
+=for Pod::Coverage BUILD
+
 =head2 sign( $request, $relaystate )
 
 Signs the given request, and returns the URL to which the user's
