@@ -7,11 +7,13 @@
 # has become more bother than it is worth.
 use strict;
 use warnings;
-package Net::SAML2::XML::Sig; use base qw(XML::Sig); 1;
-our $VERSION = '0.56'; # TRIAL VERSION
-
+package Net::SAML2::XML::Sig; use base qw(XML::Sig);
+our $VERSION = '0.58'; #TRIAL VERSION
+#
 # ABSTRACT: Net::SAML2 subclass of XML::Sig
 
+1;
+
 __END__
 
 =pod
@@ -24,7 +26,7 @@ Net::SAML2::XML::Sig - Net::SAML2 subclass of XML::Sig
 
 =head1 VERSION
 
-version 0.56
+version 0.58
 
 =head1 AUTHOR
 
